#-------------------------------------------------------------------------
# ESTIMATION CODE FOR WP 11/2019: TOO MUCH OF A GOOD THING? HOUSEHOLDS 
# MACROECONOMIC CONDITIONS AND CREDIT DYNAMICS
# 
# We focus on the link between the macroeconomic conditions faced by 
# households, the confidence of households as investors and consumers, 
# and households demand for credit. 
# We utilize Households' Macroeconomic Conditions Index (H-MCI)
# which we constructed for this purpose. 
# The analysis is on a sample of 21 OECD countries.
#
# We provide the code and the data sample in good faight. 
# The views expressed are those of the authors and do not necessarily 
# reflect the official views of the CNB.
#
# The use of the code and H-MCI is free. However, we kindly ask 
# the user to reference the following methodological papers:
#   Hodula, M., S. Malovan, and J. Frait (2019). Introducing a New Index 
#   of Households Macroeconomic Conditions. Czech National Bank Working 
#   Paper No. 10/2019.
#   Hodula, M., S. Malovan, and J. Frait (2019). Too Much of a Good Thing? 
#   Households Macroeconomic Conditions and Credit Dynamics. Czech 
#   National Bank Working Paper No. 11/2019.
#
# .........................................................................
# The code and the dataset are maintained by the authors. It is an ongoing 
# research and we welcome suggestions to enhance coverage or 
# the methodlogical approach. For questions and feedback please 
# contact simona.malovana@cnb.cz or martin.hodula@cnb.cz. 
# .........................................................................
#
# File:        panle_data_regression.R
#
# Description: This the main file for CNB WP 11/2019:
#              (1) Prepares data to be used in estimation.
#              (2) Runs the dynamic panel data regressions (mean and quantile) and save the output.
#              (3) Plots figures.
#-------------------------------------------------------------------------

rm(list=ls())
#.libPaths( c("N:/R_LIBRARY", .libPaths())) # append the new library to the existing library path

#------------------------------------------------------------------------------#
# Load required packages and source all programs to be used in the estimation.
#------------------------------------------------------------------------------#
if (!require("reshape2")) {install.packages("reshape2"); library("reshape2")} # Time series package
if (!require("mFilter")) {install.packages("mFilter"); library("mFilter")} # HP filter
if (!require("plm")) {install.packages("plm"); library("plm")} # Optimization
if (!require("stargazer")) {install.packages("stargazer"); library("stargazer")} # Time series package
if (!require("rqpd")) {install.packages("rqpd"); library("rqpd")} # HP filter
if (!require("fanplot")) {install.packages("fanplot"); library("fanplot")} # Optimization

# Set working directory
setwd("~/DP_estimation/data")

#------------------------------------------------------------------------------#
# PART 1: PREPARE DATA
#------------------------------------------------------------------------------#
#
# A. CREDIT VARIABLES
#
# Load and merge data samples
credit <- read.csv("credit.csv", sep=";") # credit to households 
creditgdp <- read.csv("creditgdp.csv", sep=";") # credit to housheold over GDP
n <- length(credit[1,]) - 1 # no. of countries
t <- length(credit[,1]) # no. of time periods
data <- cbind(melt(credit, id.vars = c("time")), melt(creditgdp, id.vars = c("time"))[,3])
data$time0 <- rep(1:t,n)
colnames(data) <- c("time","country","credit","creditgdp","time0")

# Discart some early periods of higher volatility
# Prior 2005
for (i in c("CZ","GR","HU","PL")){
  data$credit[which(as.character(data$country)==i&data$time0<60)] <- NA 
  data$creditgdp[which(as.character(data$country)==i&data$time0<60)] <- NA
}
# Prior 2000
for (i in c("NL","ES","PT")){
  data$credit[which(as.character(data$country)==i&data$time0<40)] <- NA 
  data$creditgdp[which(as.character(data$country)==i&data$time0<40)] <- NA
}

# HP filter
lambda <- 26000

data$trendcredit <- NA
data$cyclecredit <- NA
data$driftcredit <- NA
data$trendcreditgdp <- NA
data$cyclecreditgdp <- NA
data$driftcreditgdp <- NA

for (i in unique(as.character(data$country))){
  hp.1 <- hpfilter(data$creditgdp[which(as.character(data$country)==i & data$creditgdp>0)], freq=lambda, type="lambda", drift=TRUE)
  hp.2 <- hpfilter(data$credit[which(as.character(data$country)==i & data$credit>0)], freq=lambda, type="lambda", drift=TRUE)
  
  data$trendcreditgdp[which(as.character(data$country)==i & data$creditgdp>0)] <- hp.1$trend
  data$cyclecreditgdp[which(as.character(data$country)==i & data$creditgdp>0)] <- hp.1$cycle
  data$driftcreditgdp[which(as.character(data$country)==i & data$creditgdp>0)] <- data$creditgdp[which(as.character(data$country)==i & data$creditgdp>0)] - hp.1$cycle - hp.1$trend
  
  data$trendcredit[which(as.character(data$country)==i & data$credit>0)] <- hp.2$trend
  data$cyclecredit[which(as.character(data$country)==i & data$credit>0)] <- hp.2$cycle
  data$driftcredit[which(as.character(data$country)==i & data$credit>0)] <- data$credit[which(as.character(data$country)==i & data$credit>0)] - hp.2$cycle - hp.2$trend
}
data$creditgap <- data$cyclecredit / (data$trendcredit + data$driftcredit) * 100
data$creditgdpgap <- data$cyclecreditgdp / (data$trendcreditgdp + data$driftcreditgdp) * 100

# Discart Norway, data on CCI not available
data <- data[-which(as.character(data$country)=="NO"),]

# Save credit gaps
write.csv(data, "creditgaps.csv")

#
# B. CONTROLS, HMCI AND CCI
#
# Load additional variables
controls <- read.csv("controls.csv", sep=";") 
hmci <- read.csv("hmcigrowth.csv", sep=";") 
hmciGap <- read.csv("hmcigap.csv", sep=";") 
dummies <- read.csv("dummies.csv", sep=";") 
cci <- read.csv("cci.csv", sep=";")

# Reshape 
cci <- melt(cci, by="time")
hmci <- melt(hmci, id.vars = c("time"))
hmciGap <- melt(hmciGap, id.vars = c("time"))

# Assign column names
colnames(cci) <- c("time", "country", "cci")
colnames(hmci) <- c("time", "country", "hmci")
colnames(hmciGap) <- c("time", "country", "hmciGap")

# Create time variables
time <- unique(as.character(cci$time))
time2 <- unique(as.character(hmci$time))
tempTime <- (1+length(time2)-length(time)):length(time2)
tempTime2 <- 1:length(time2)
cci$time0 <- rep(tempTime, length(unique(cci$country)))
dummies$time0 <- rep(tempTime, length(unique(dummies$country)))
hmci$time0 <- rep(tempTime2, length(unique(hmci$country)))
hmciGap$time0 <- rep(tempTime2, length(unique(hmciGap$country)))

# Sorting variable
hmci$sort <- 1:length(hmci$time)

# Create ids
dummies$id <- paste(dummies$country,dummies$time0)
cci$id <- paste(cci$country,cci$time0)
hmci$id <- paste(hmci$country,hmci$time0)
hmciGap$id <- paste(hmciGap$country,hmciGap$time0)
controls$id <- paste(controls$country,controls$time0)
data$id <- paste(data$country,data$time0)

# Merge subsamples 
data1 <- merge(cci, dummies, by="id")
data2 <- merge(hmci, hmciGap, by="id")
data3 <- merge(data1, data, by="id")
data <- merge(data2, data3, by="id")

# Sort data frame, discart redundant columns and assign column names
data <- data[order(data$sort),]
data <- data[,-c(1,6,7,8,10,11,12,14,15,16,21,22,23,26)]
colnames(data) <- c("time","country","hmci","time0","hmciGap","cci","oecd","imfdeveloped","eu","ea","credit","creditgdp","trendcredit","cyclecredit","driftcredit","trendcreditgdp","cyclecreditgdp","driftcreditgdp","creditgap","creditgdpgap")

# Create panel structure
data <- pdata.frame(data, index=c("country", "time0"))

# Create yoy credit growth
data$yoy3 <- diff(data$credit, lag = 4) / c(rep(NA,4), data$credit[1:(length(data$credit)-4)]) * 100

#
data[which(data$yoy3 > 70),] <- NA
data[which(data$creditgdpgap < (-30)),] <- NA

# Create dummy variables
data$dummy1 <- as.numeric(data$hmci > 0)
data$ccidummy <- as.numeric(data$cci > 0)
data$dummy2 <- data$ccidummy*data$dummy1 # both above zero
data$dummy3 <- (1-data$ccidummy)*(1-data$dummy1) # both below zero
data$dummy4 <- (1-(data$dummy2+data$dummy3)) # one is below and one is abow zero

data$hmci1 <- data$hmci*data$dummy1
data$hmci2 <- data$hmci*(1-data$dummy1)

# Country groups
data$hmciAE <- data$hmci*data$imfdeveloped
data$hmciEME <- data$hmci*(1-data$imfdeveloped)

data$cciAE <- data$cci*data$imfdeveloped
data$cciEME <- data$cci*(1-data$imfdeveloped)

data$hmciAE1 <- data$hmci*data$imfdeveloped*data$dummy1
data$hmciAE2 <- data$hmci*data$imfdeveloped*(1-data$dummy1)

data$hmciEME1 <- data$hmci*(1-data$imfdeveloped)*data$dummy1
data$hmciEME2 <- data$hmci*(1-data$imfdeveloped)*(1-data$dummy1)

data$hmcicci1 <- data$hmci*data$dummy2
data$hmcicci2 <- data$hmci*data$dummy4
data$hmcicci3 <- data$hmci*data$dummy3

data$hmcicci1ae <- data$hmcicci1*data$imfdeveloped
data$hmcicci2ae <- data$hmcicci2*data$imfdeveloped
data$hmcicci3ae <- data$hmcicci3*data$imfdeveloped

data$hmcicci1eme <- data$hmcicci1*(1-data$imfdeveloped)
data$hmcicci2eme <- data$hmcicci2*(1-data$imfdeveloped)
data$hmcicci3eme <- data$hmcicci3*(1-data$imfdeveloped)

# Create shorter data sample with controls
data$sort <- 1:length(data$time)
data$id <- paste(data$country,data$time0)
data_shorter <- merge(data, controls, by="id")
data_shorter <- data_shorter[order(data_shorter$sort),]
data_shorter <- data_shorter[,-c(1,58:61)]
colnames(data_shorter) <- c("time","country",colnames(data_shorter)[3],"time0",colnames(data_shorter)[5:54])
data_longer <- data

# Save both samples
write.csv(data, "data.csv")
write.csv(data, "data_shorter.csv")

#------------------------------------------------------------------------------#
# PART 2: ESTIMATE DYNAMIC PANEL DATA MODEL
#------------------------------------------------------------------------------#
#
# A. MEAN REGRESSION WITHOUT CONTROLS
#
setwd("~/DP_estimation/results")
#
models <- list(yoy3 ~ lag(yoy3) + lag(hmci,1), 
  yoy3 ~ lag(yoy3) + lag(hmciAE,1) + lag(hmciEME,1),
  yoy3 ~ lag(yoy3) + lag(hmciAE1,1) + lag(hmciAE2,1) + lag(hmciEME1,1) + lag(hmciEME2,1),
  creditgap  ~ lag(creditgap,1) + lag(hmci,1),
  creditgap  ~ lag(creditgap,1) + lag(hmciAE,1) + lag(hmciEME,1),
  creditgap  ~ lag(creditgap,1) + lag(hmciAE1,1) + lag(hmciAE2,1) + lag(hmciEME1,1) + lag(hmciEME2,1),
  creditgdpgap  ~ lag(creditgdpgap,1) + lag(hmci,1),
  creditgdpgap  ~ lag(creditgdpgap,1) + lag(hmciAE,1) + lag(hmciEME,1),
  creditgdpgap  ~ lag(creditgdpgap,1) + lag(hmciAE1,1) + lag(hmciAE2,1) + lag(hmciEME1,1) + lag(hmciEME2,1))

res <- list()
for (i in 1:9){
  if (i < 4){
    res[[i]] <- list(plm(models[[i]], data=data[-c(which(data$yoy3==Inf), which(is.na(data$yoy3)==TRUE), which(is.na(data$hmci)==TRUE)),],
                  index=c("country", "time0"), model="within"))
  } else if (i > 3 & i < 7){
    res[[i]] <- list(plm(models[[i]], data=data[-c(which(data$creditgap==Inf), which(is.na(data$creditgap)==TRUE), which(is.na(data$hmci)==TRUE)),],
                         index=c("country", "time0"), model="within"))
  } else if (i > 6){
    res[[i]] <- list(plm(models[[i]], data=data[-c(which(data$creditgdpgap==Inf), which(is.na(data$creditgdpgap)==TRUE), which(is.na(data$hmci)==TRUE)),],
                         index=c("country", "time0"), model="within"))
  }
}
stargazer(res, out="res1.htm")

#
models <- list(yoy3 ~ lag(yoy3) + lag(hmcicci1,1) + lag(hmcicci2,1)  + lag(hmcicci3,1),
               yoy3 ~ lag(yoy3) + lag(hmcicci1ae,1) + lag(hmcicci2ae,1)  + lag(hmcicci3ae,1) + lag(hmcicci1eme,1) + lag(hmcicci2eme,1)  + lag(hmcicci3eme,1),
               creditgap ~ lag(creditgap) + lag(hmcicci1,1) + lag(hmcicci2,1)  + lag(hmcicci3,1),
               creditgap ~ lag(creditgap) + lag(hmcicci1ae,1) + lag(hmcicci2ae,1)  + lag(hmcicci3ae,1) + lag(hmcicci1eme,1) + lag(hmcicci2eme,1)  + lag(hmcicci3eme,1),
               creditgdpgap ~ lag(creditgdpgap) + lag(hmcicci1,1) + lag(hmcicci2,1)  + lag(hmcicci3,1),
               creditgdpgap ~ lag(creditgdpgap) + lag(hmcicci1ae,1) + lag(hmcicci2ae,1)  + lag(hmcicci3ae,1) + lag(hmcicci1eme,1) + lag(hmcicci2eme,1)  + lag(hmcicci3eme,1))
               
res <- list()
for (i in 1:6){
  if (i < 3){
    res[[i]] <- list(plm(models[[i]], data=data[-c(which(data$yoy3==Inf), which(is.na(data$yoy3)==TRUE), which(is.na(data$hmci)==TRUE)),],
                         index=c("country", "time0"), model="within"))
  } else if (i > 2 & i < 5){
    res[[i]] <- list(plm(models[[i]], data=data[-c(which(data$creditgap==Inf), which(is.na(data$creditgap)==TRUE), which(is.na(data$hmci)==TRUE)),],
                         index=c("country", "time0"), model="within"))
  } else if (i > 4){
    res[[i]] <- list(plm(models[[i]], data=data[-c(which(data$creditgdpgap==Inf), which(is.na(data$creditgdpgap)==TRUE), which(is.na(data$hmci)==TRUE)),],
                         index=c("country", "time0"), model="within"))
  }
}
stargazer(res, out="res2.htm")

#
# B. MEAN REGRESSION WITH CONTROLS
#
data <- data_shorter
#
models <- list(yoy3 ~ lag(yoy3) + lag(hmci,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1), 
               yoy3 ~ lag(yoy3) + lag(hmciAE,1) + lag(hmciEME,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1),
               yoy3 ~ lag(yoy3) + lag(hmciAE1,1) + lag(hmciAE2,1) + lag(hmciEME1,1) + lag(hmciEME2,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1),
               creditgap  ~ lag(creditgap,1) + lag(hmci,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1),
               creditgap  ~ lag(creditgap,1) + lag(hmciAE,1) + lag(hmciEME,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1),
               creditgap  ~ lag(creditgap,1) + lag(hmciAE1,1) + lag(hmciAE2,1) + lag(hmciEME1,1) + lag(hmciEME2,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1),
               creditgdpgap  ~ lag(creditgdpgap,1) + lag(hmci,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1),
               creditgdpgap  ~ lag(creditgdpgap,1) + lag(hmciAE,1) + lag(hmciEME,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1),
               creditgdpgap  ~ lag(creditgdpgap,1) + lag(hmciAE1,1) + lag(hmciAE2,1) + lag(hmciEME1,1) + lag(hmciEME2,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1))

res <- list()
for (i in 1:9){
  if (i < 4){
    res[[i]] <- list(plm(models[[i]], data=data[-c(which(data$yoy3==Inf), which(is.na(data$yoy3)==TRUE), which(is.na(data$hmci)==TRUE)),],
                         index=c("country", "time0"), model="within"))
  } else if (i > 3 & i < 7){
    res[[i]] <- list(plm(models[[i]], data=data[-c(which(data$creditgap==Inf), which(is.na(data$creditgap)==TRUE), which(is.na(data$hmci)==TRUE)),],
                         index=c("country", "time0"), model="within"))
  } else if (i > 6){
    res[[i]] <- list(plm(models[[i]], data=data[-c(which(data$creditgdpgap==Inf), which(is.na(data$creditgdpgap)==TRUE), which(is.na(data$hmci)==TRUE)),],
                         index=c("country", "time0"), model="within"))
  }
}
stargazer(res, out="res3.htm")

#
models <- list(yoy3 ~ lag(yoy3) + lag(hmcicci1,1) + lag(hmcicci2,1) + lag(hmcicci3,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1),
               yoy3 ~ lag(yoy3) + lag(hmcicci1ae,1) + lag(hmcicci2ae,1)  + lag(hmcicci3ae,1) + lag(hmcicci1eme,1) + lag(hmcicci2eme,1)  + lag(hmcicci3eme,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1),
               creditgap ~ lag(creditgap) + lag(hmcicci1,1) + lag(hmcicci2,1)  + lag(hmcicci3,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1),
               creditgap ~ lag(creditgap) + lag(hmcicci1ae,1) + lag(hmcicci2ae,1)  + lag(hmcicci3ae,1) + lag(hmcicci1eme,1) + lag(hmcicci2eme,1) + lag(hmcicci3eme,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1),
               creditgdpgap ~ lag(creditgdpgap) + lag(hmcicci1,1) + lag(hmcicci2,1)  + lag(hmcicci3,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1),
               creditgdpgap ~ lag(creditgdpgap) + lag(hmcicci1ae,1) + lag(hmcicci2ae,1)  + lag(hmcicci3ae,1) + lag(hmcicci1eme,1) + lag(hmcicci2eme,1) + lag(hmcicci3eme,1) + lag(ROA,1) + lag(NPL,1) + lag(CA,1))

res <- list()
for (i in 1:6){
  if (i < 3){
    res[[i]] <- list(plm(models[[i]], data=data[-c(which(data$yoy3==Inf), which(is.na(data$yoy3)==TRUE), which(is.na(data$hmci)==TRUE)),],
                         index=c("country", "time0"), model="within"))
  } else if (i > 2 & i < 5){
    res[[i]] <- list(plm(models[[i]], data=data[-c(which(data$creditgap==Inf), which(is.na(data$creditgap)==TRUE), which(is.na(data$hmci)==TRUE)),],
                         index=c("country", "time0"), model="within"))
  } else if (i > 4){
    res[[i]] <- list(plm(models[[i]], data=data[-c(which(data$creditgdpgap==Inf), which(is.na(data$creditgdpgap)==TRUE), which(is.na(data$hmci)==TRUE)),],
                         index=c("country", "time0"), model="within"))
  }
}
stargazer(res, out="res4.htm")

#
# C. QUANTILE REGRESSION WITHOUT CONTROLS
#
data <- data_longer
siglev <- c(0.1, 0.05, 0.01) # significance levels: 10%, 5%, 1%
#
# Lags
data$yoy3_l <- c(NA,data$yoy3[-length(data$yoy3)]) # create a new var with data displaced by 1 unit
data$yoy3_l[c(data$country) != c(NA, data$country[-length(data$country)])] <- NA # NA data with different current and lagged id.
data$creditgap_l <- c(NA,data$creditgap[-length(data$creditgap)])
data$creditgap_l[c(data$country) != c(NA, data$country[-length(data$country)])] <- NA
data$creditgdpgap_l <- c(NA,data$creditgdpgap[-length(data$creditgdpgap)])
data$creditgdpgap_l[c(data$country) != c(NA, data$country[-length(data$country)])] <- NA
data$hmciAE_l <- c(NA,data$hmciAE[-length(data$hmciAE)])
data$hmciAE_l[c(data$country) != c(NA, data$country[-length(data$country)])] <- NA
data$hmciEME_l <- c(NA,data$hmciEME[-length(data$hmciEME)])
data$hmciEME_l[c(data$country) != c(NA, data$country[-length(data$country)])] <- NA
data$hmci1_l <- c(NA,data$hmci1[-length(data$hmci1)])
data$hmci1_l[c(data$country) != c(NA, data$country[-length(data$country)])] <- NA
data$hmci2_l <- c(NA,data$hmci2[-length(data$hmci2)])
data$hmci2_l[c(data$country) != c(NA, data$country[-length(data$country)])] <- NA

models <- list(yoy3 ~ yoy3_l + hmci1_l + hmci2_l | country,
              creditgap ~ creditgap_l + hmci1_l + hmci2_l | country,
              creditgdpgap ~ creditgdpgap_l + hmci1_l + hmci2_l | country,
              yoy3 ~ yoy3_l + hmciAE_l + hmciEME_l | country,
              creditgap ~ creditgap_l + hmciAE_l + hmciEME_l | country,
              creditgdpgap ~ creditgdpgap_l + hmciAE_l + hmciEME_l | country)
q <- 4 # number of quantiles
c <- 3+1 # number of coefficients (k regressors + intercept)
N <- (c*q)/q

res <- list()
stars <- list()
coeff <- list()
se <- list()
pval <- list()
upperCI <- list()
lowerCI <- list()
for (i in 1:6){
  if (i==1 || i==4){
    res[[i]] <- list(rqpd(models[[i]], panel(taus = c(0.2, 0.4, 0.6, 0.8), tauw = rep(1/4, 4),lambda = 5), data = data[-c(which(data$yoy3==Inf), which(is.na(data$yoy3)==TRUE), which(is.na(data$hmci)==TRUE)),]))
  } else if (i==2 || i==5){
    res[[i]] <- list(rqpd(models[[i]], panel(taus = c(0.2, 0.4, 0.6, 0.8), tauw = rep(1/4, 4),lambda = 5), data = data[-c(which(data$creditgap==Inf), which(is.na(data$creditgap)==TRUE), which(is.na(data$hmci)==TRUE)),]))
  } else if (i==3 || i==6){
    res[[i]] <- list(rqpd(models[[i]], panel(taus = c(0.2, 0.4, 0.6, 0.8), tauw = rep(1/4, 4),lambda = 5), data = data[-c(which(data$creditgdpgap==Inf), which(is.na(data$creditgdpgap)==TRUE), which(is.na(data$hmci)==TRUE)),]))
  } 
  fit <- summary(res[[i]][[1]])
  str <- sapply(fit$coefficients[,4], function(x) {ifelse(x < siglev[3], "***", ifelse(x < siglev[2], "**", ifelse(x < siglev[1], "*", "")))})
  stars[[i]] <- matrix(str[1:(c*q)],N)
  coeff[[i]] <- matrix(fit$coefficients[1:(c*q),1],N)
  se[[i]] <- matrix(fit$coefficients[1:(c*q),2],N)
  pval[[i]] <- matrix(fit$coefficients[1:(c*q),4],N)
  upperCI[[i]] <- coeff[[i]] + 1.645*se[[i]]
  lowerCI[[i]] <- coeff[[i]] - 1.645*se[[i]]
}

# Plot results
names1 <- c("H-MCI > 0","H-MCI < 0")
names2 <- c("AEs","EMEs")
x <- 2
j <- 1
y <- 1
ncoeff <- x*y

for (ii in 1:6){
  cf <- coeff[[ii]]
  uCI <- upperCI[[ii]]
  lCI <- lowerCI[[ii]]
  #
  par(xpd=FALSE)
  par(mar=c(3.5,3.5,3,0.1))
  par(mfrow=c(y,x))
  j <- 1
  for (i in 3:4){
    var <- ts(cbind(cf[i,],uCI[i,],lCI[i,]),start=0.2,frequency=5)
    u <- round(max(var[,2])+0.2*max(var[,2]),1)
    l <- round(min(var[,3])-0.2*abs(min(var[,3])),1)
    plot(var[,1], type = "l", col = "black", lwd = 2,
         ylim = c(l,u),
         xaxt = "n", yaxt = "n", ylab="", xlab="")
    fan(t(var[,2:3]), data.type = "values", start=0.2,frequency=5,
        fan.col = colorRampPalette(c("gray80")), probs = c(0,1),
        ln = NULL, rlab = NULL)
    lines(var[,1], lty = 1, col = "black", lwd = 2)
    if (ii < 4){
      title(main=names1[j])
    } else {
      title(main=names2[j])
    }
    j <- j + 1
    # aesthetics
    if (abs(l)+abs(u)>2){
      axis(4, at = seq(-10,10,0.5), las = 2, tcl = 0.5, labels = FALSE)
      axis(2, at = seq(-10,10,0.5), las = 2, tcl = 0.5) 
    } else if (abs(l)+abs(u)>1){
      axis(4, at = seq(-2,2,0.2), las = 2, tcl = 0.5, labels = FALSE)
      axis(2, at = seq(-2,2,0.2), las = 2, tcl = 0.5) 
    }
    else{
      axis(4, at = seq(-2,2,0.1), las = 2, tcl = 0.5, labels = FALSE)
      axis(2, at = seq(-2,2,0.1), las = 2, tcl = 0.5)
    }
    axis(1, at = seq(0,1,0.2), tcl = 0.5)
    abline(h=0)
  }
}

# Output tables
tables <-list(matrix(NA, ncol=q, nrow=8),
              matrix(NA, ncol=q, nrow=8),
              matrix(NA, ncol=q, nrow=8),
              matrix(NA, ncol=q, nrow=8),
              matrix(NA, ncol=q, nrow=8),
              matrix(NA, ncol=q, nrow=8))

for (ii in 1:6){
  cf <- coeff[[ii]]
  str <- stars[[ii]]
  s <- se[[ii]]
  for (i in 1:q){
    tables[[ii]][c(1,3,5,7),i] <- paste(round(cf[,i],3), str[,i], sep="")
    tables[[ii]][c(2,4,6,8),i] <- paste("(", round(s[,i],3), ")", sep="")
  }
}
stargazer(tables, out="resQ.htm") 


#------------------------------------------------------------------------------#
# PART 3: PLOT FIGURES
#------------------------------------------------------------------------------#
setwd("~/DP_estimation/data")
#
data <- read.csv("creditgaps.csv", sep=",")
#
# [800x900]
# (1)
par(mar=c(2,2,2,2))
par(mfrow=c(7,3))
for(i in unique(data$country)){
  temp <- data$creditgap[which(data$country==i)]
  plot(ts(data$creditgap[which(data$country==i)],start=c(1990,1),frequency=4), ylim=c(min(temp,na.rm=TRUE),max(temp,na.rm=TRUE)), 
       lwd=2, main=i, ylab="", xlab="")
  par(new = TRUE)
  temp <- data$creditgdpgap[which(data$country==i)]
  plot(ts(data$creditgdpgap[which(data$country==i)],start=c(1990,1),frequency=4), ylim=c(min(temp,na.rm=TRUE),max(temp,na.rm=TRUE)), 
       lwd=2, main=i, xaxt="n", yaxt="n", ylab="", xlab="", col="red")
  axis(4, at=seq(-40,40,5))
  abline(h=0)
}
# legend
par(fig = c(0, 1, 0, 1), oma = c(0,0,0,0), mar = c(0, 0, 0, 0), new = TRUE)
plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
legend("bottom", legend=c("Credit gap","Credit-to-GDP gap (rhs)"), 
      col=c("black","red"), lty=1, lwd=2,
      xpd = TRUE, horiz = TRUE, inset = c(0,0.01), bty = "n")

# (2)
par(mar=c(2,2,2,2))
par(mfrow=c(7,3))
for(i in unique(data$country)){
  temp <- data$credit[which(data$country==i)]
  plot(ts(data$credit[which(data$country==i)],start=c(1990,1),frequency=4), ylim=c(min(temp,na.rm=TRUE),max(temp,na.rm=TRUE)), 
       lwd=2, main=i, ylab="", xlab="")
  par(new = TRUE)
  temp <- data$creditgdp[which(data$country==i)]
  plot(ts(data$creditgdp[which(data$country==i)],start=c(1990,1),frequency=4), ylim=c(min(temp,na.rm=TRUE),max(temp,na.rm=TRUE)), 
       lwd=2, main=i, xaxt="n", yaxt="n", ylab="", xlab="", col="red")
  axis(4, at=seq(0,400,5))
  abline(h=0)
}
# legend
par(fig = c(0, 1, 0, 1), oma = c(0,0,0,0), mar = c(0, 0, 0, 0), new = TRUE)
plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
legend("bottom", legend=c("Credit","Credit-to-GDP ratio (rhs)"), 
      col=c("black","red"), lty=1, lwd=2,
      xpd = TRUE, horiz = TRUE, inset = c(0,0.01), bty = "n")

#
# [1040 x 500]
# (3)
data <- read.csv("data.csv", sep=",")
#
par(mfrow=c(1,2))
par(mar=c(5,5,3,3))
plot(cbind(data$hmci, data$yoy3), pch=21,
     col=rainbow(22)[data$country], bg=rainbow(22)[data$country],
     ylab = "Annual credit growth", xlab="H-MCI")
abline(plm(yoy3 ~ hmci, data=data[-c(which(data$imfdeveloped==0), which(data$yoy3==Inf), which(is.na(data$yoy3)==TRUE), which(is.na(data$hmci)==TRUE)),], 
           index=c("country", "time"), model="within"), col="red", lwd=2)
abline(plm(yoy3 ~ hmci, data=data[-c(which(data$imfdeveloped==1), which(data$yoy3==Inf), which(is.na(data$yoy3)==TRUE), which(is.na(data$hmci)==TRUE)),], 
           index=c("country", "time"), model="within"), 
       col="blue", lwd=2)
abline(h=0, col="gray70")
abline(v=0, col="gray70")
#
plot(cbind(data$hmci, data$creditgdpgap), pch=21,
     col=rainbow(22)[data$country], bg=rainbow(22)[data$country],
     ylab = "Credit-to-GDP gap", xlab="H-MCI")
abline(plm(creditgdpgap ~ hmci, data=data[-c(which(data$imfdeveloped==0), which(data$creditgdpgap==Inf), which(is.na(data$creditgdpgap)==TRUE), which(is.na(data$hmci)==TRUE)),], 
           index=c("country", "time"), model="within"), col="red", lwd=2)
abline(plm(creditgdpgap ~ hmci, data=data[-c(which(data$imfdeveloped==1), which(data$creditgdpgap==Inf), which(is.na(data$creditgdpgap)==TRUE), which(is.na(data$hmci)==TRUE)),], 
           index=c("country", "time"), model="within"), 
       col="blue", lwd=2)
abline(h=0, col="gray70")
abline(v=0, col="gray70")
